using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.Classes;
using Luski.GUI.MainScreen.UI.LuskiControls;
using Luski.Interfaces;
using Luski.net;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.GraphicsLibraryFramework;
using Rectangle = GraphicsManager.Objects.Rectangle;

namespace Luski.GUI.MainScreen.UI;

public class AddServerOverlayld : UserControl, IServerOverlay
{
    private UserControl Form, btn;
    private DropDown<VersionDropButton> version;

    public AccountButton? Selected { get; set; }
    public UserControl page { get; set; }
    private TextBox? UserName, Password, DisplayName, tb;
    
    private Rectangle? rec;
    
    public AddServerOverlayld()
    {
        base.Size = Globals.ms.ClientSize;
        base.BackgroundColor = new(0, 0, 0, 130);
        Anchor = ObjectAnchor.All;
        
        Form = new(Globals.ms.TextureManager.GetTextureResource("RoundedRectangle.png"))
        {
            Size = new(350.ScaleInt(), 347.ScaleInt()),
            BackgroundColor = new(32,32,32,255),
            Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            TextureDisplay = TextureDisplay.Center
        };
        Label t;
        Form.Controls.Add(t=new Label(Globals.DefaultFont) { Scale = 1.6f, Text = "Add Server", Color = Globals.DodgerBlue });
        t.Location = new((Form.Size.X / 2) - (t.Size.X / 2), t.Location.Y);
        
        #region Server Loc
        Label? ll = new Label(Globals.DefaultFont)
        {
            Text = net.API.DefaultVersion,
            Color = Color4.DarkGray,
            IgnoreHover = true
        };
        Vector2i s =new((int)(ll.Size.X * 1.6f),27.ScaleInt());
        ll = null;
        
        Rectangle line = new Rectangle()
        {
            Size = new Vector2i(1.ScaleInt()),
            BackgroundColor = Globals.DodgerBlue
        };

        tb = new()
        {
            Location = new(10.ScaleInt(),50.ScaleInt()),
            Size = s,
            WatermarkText = "Server Address",
            TextLocation = TextLocation.LineCenter,
            AllowMultiLine = false
        };
        tb.Textures[0] = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
        tb.LetterPress += () =>
        {
            Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
            if (tb.Textures[0].handel != good.handel)
            {
                tb.Textures[0] = good;
                if (UserName!.Textures[0].handel == good.handel &&
                    Password!.Textures[0].handel == good.handel &&
                    (rec is null || rec.Textures.Count > 1) &&
                    btn!.Textures[0].handel != good.handel &&
                    tb.Textures[0].handel == good.handel)
                {
                    btn.Textures[0] = good;
                }
                Globals.ms.TryDraw();
            }
            return Task.CompletedTask;
        };
        tb.LetterRemoved += () =>
        {
            Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
            if (tb.Text.Length == 0 && tb.Textures[0].handel != bad.handel)
            {
                tb.Textures[0] = bad;
                btn!.Textures[0] = bad;
                Globals.ms.TryDraw();
            }
                    
            return Task.CompletedTask;
        };
        tb.KeyPress += args =>
        {
            if (args.Key == Keys.Tab && !args.Shift)
            {
                UserName!.Focus();
            }

            return Task.CompletedTask;
        };
       
        tb.SetSize(Form.Size.X - tb.Location.X - tb.Location.X - tb.Location.X - s.X, tb.Size.Y);
        Form.Controls.Add(tb);
        version = new DropDown<VersionDropButton>(Form.Textures[0], line)
        {
            DropDownParentOverride = Form,
            Size = s,
            BackgroundColor = new(255,20,20,255),
            TextureDisplay = TextureDisplay.HorizontalCenter,
            Shader = Form.Shader,
            Location = new(tb.Location.X + tb.Size.X + tb.Location.X, tb.Location.Y)
        };
        foreach (string v in Globals.Luski.SupportedVersions)
        {
            VersionDropButton option = new VersionDropButton(s, v)
            {
                LoadDisplay = () =>
                {
                    try
                    {
                        Label ll2 = new Label(Globals.DefaultFont)
                        {
                            Text = v,
                            Color = Color4.DarkGray,
                            IgnoreHover = true,

                        };
                        ll2.Location = new((version.Size.X / 2) - (ll2.Size.X / 2),
                            ((version.Size.Y - ll2.Size.Y) / 2));
                        version.Controls.Add(ll2);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e);
                    }
                }
            };
            version.AddOption(option);
            if (v == net.API.DefaultVersion)
            {
                version.SetSelected(option);
            }
        }
        
        version.OpenStatusChanged += VersionOnOpenStatusChanged;
        version.OptionSelected += VersionOnOptionSelected;
        
        
        version.DropDownContainer.Textures.Add(Globals.ms.TextureManager.GetTextureResource("RoundedRectangleBottom.png"));
        version.DropDownContainer.TextureDisplay = TextureDisplay.Center;
        version.DropDownContainer.Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context];
        version.DropDownContainer.BackgroundColor = version.BackgroundColor;
        version.OpenStatusChanged += b =>
        {
            BlockDraw = true;
            if (b)
            {
                version.Textures[0] = Globals.ms.TextureManager.GetTextureResource("RoundedRectangleTop.png");
            }
            else
            {
                version.Textures[0] = Globals.ms.TextureManager.GetTextureResource("RoundedRectangle.png");
            }

            BlockDraw = false;
            return Task.CompletedTask;
        };
        
        
        #endregion

        #region Buttons

        AccountButton ca = new AccountButton("Create Account", this)
        {
            Location = new(tb.Location.X, tb.Location.Y + tb.Location.X + tb.Size.Y),
            OnPageLoad = () =>
            {
                btn!.Textures[0] = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                page!.Controls.Add(UserName = new()
                {
                    Location = new(0, 10.ScaleInt()),
                    Size = new(page.Size.X, 30.ScaleInt()),
                    WatermarkText = "Username",
                    TextLocation = TextLocation.LineCenter,
                    AllowMultiLine = false
                });
                UserName.Textures[0] = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                UserName.LetterPress += () =>
                {
                    Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
                    if (UserName.Textures[0].handel != good.handel)
                    {
                        UserName.Textures[0] = good;
                        if (Password!.Textures[0].handel == good.handel && DisplayName!.Textures[0].handel == good.handel && rec!.Textures.Count > 1 && btn!.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
                        {
                            btn.Textures[0] = good;
                        }
                        Globals.ms.TryDraw();
                    }
                    return Task.CompletedTask;
                };
                UserName.LetterRemoved += () =>
                {
                    Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
                    if (UserName.Text.Length == 0 && UserName.Textures[0].handel != bad.handel)
                    {
                        UserName.Textures[0] = bad;
                        btn.Textures[0] = bad;
                        Globals.ms.TryDraw();
                    }
                    
                    return Task.CompletedTask;
                };
                UserName.KeyPress += args =>
                {
                    if (args.Key == Keys.Tab)
                    {
                        if (args.Shift)
                        {
                            tb.Focus();
                        }
                        else
                        {
                            Password!.Focus();
                        }
                    }

                    return Task.CompletedTask;
                };
                
                page.Controls.Add(Password = new()
                {
                    Location = new(0, UserName.Location.Y + UserName.Size.Y + UserName.Location.Y + UserName.Location.Y),
                    Size = new(page.Size.X, UserName.Size.Y),
                    WatermarkText = "Password",
                    TextLocation = TextLocation.LineCenter,
                    AllowMultiLine = false,
                    PasswordChar = '●'
                });
                Password.Textures[0] = UserName.Textures[0];
                Password.LetterPress += () =>
                {
                    Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
                    if (Password.Textures[0].handel != good.handel)
                    {
                        Password.Textures[0] = good;
                        if (UserName!.Textures[0].handel == good.handel && DisplayName!.Textures[0].handel == good.handel && rec!.Textures.Count > 1 && btn!.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
                        {
                            btn.Textures[0] = good;
                        }
                        Globals.ms.TryDraw();
                    }
                    return Task.CompletedTask;
                };
                Password.LetterRemoved += () =>
                {
                    Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
                    if (Password.Text.Length == 0 && Password.Textures[0].handel != bad.handel)
                    {
                        Password.Textures[0] = bad;
                        btn.Textures[0] = bad;
                        Globals.ms.TryDraw();
                    }
                    
                    return Task.CompletedTask;
                };
                Password.KeyPress += args =>
                {
                    if (args.Key == Keys.Tab)
                    {
                        if (args.Shift)
                        {
                            UserName.Focus();
                        }
                        else
                        {
                            DisplayName!.Focus();
                        }
                    }

                    return Task.CompletedTask;
                };
                page.Controls.Add(rec = new(Globals.ms.TextureManager.GetAlphaCircle())
                {
                    Size = new(50.ScaleInt()),
                    BackgroundColor = Color4.Red,
                    Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
                    IgnoreHover = false
                });
                rec.Location = new(page.Size.X - rec.Size.X, page.Size.Y - rec.Size.Y);
                
                page.Controls.Add(DisplayName = new()
                {
                    Location = new(0, Password.Location.Y + Password.Size.Y + UserName.Location.Y + UserName.Location.Y),
                    Size = new(page.Size.X- tb.Location.X - rec.Size.X, Password.Size.Y ),
                    WatermarkText = "Display Name",
                    TextLocation = TextLocation.LineCenter,
                    AllowMultiLine = false
                });
                DisplayName.LetterPress += () =>
                {
                    Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
                    if (DisplayName.Textures[0].handel != good.handel)
                    {
                        DisplayName.Textures[0] = good;
                        if (UserName!.Textures[0].handel == good.handel && Password!.Textures[0].handel == good.handel && rec!.Textures.Count > 1 && btn!.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
                        {
                            btn.Textures[0] = good;
                        }
                        Globals.ms.TryDraw();
                    }
                    return Task.CompletedTask;
                };
                DisplayName.LetterRemoved += () =>
                {
                    Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
                    if (DisplayName.Text.Length == 0 && DisplayName.Textures[0].handel != bad.handel)
                    {
                        DisplayName.Textures[0] = bad;
                        btn.Textures[0] = bad;
                        Globals.ms.TryDraw();
                    }
                    
                    return Task.CompletedTask;
                };
                DisplayName.KeyPress += args =>
                {
                    if (args.Key == Keys.Tab && args.Shift)
                    {
                        Password.Focus();
                    }

                    return Task.CompletedTask;
                };
                rec.FilesDroped += RecOnFilesDroped;
                DisplayName.Textures[0] = UserName.Textures[0];
                rec.ForceDistanceUpdate(page);
                tb.Focus();
                Globals.ms.TryDraw();
            }
        };
        ca.Size = new((Form.Size.X - tb.Location.X - tb.Location.X - (tb.Location.X / 2)) / 2, ca.Size.Y);
        ca.l.Location = new((ca.Size.X - ca.l.Size.X) / 2,
            ((ca.Size.Y - ca.l.Size.Y) / 2));
        ca.l.ForceDistanceUpdate(ca);
        Form.Controls.Add(ca);
        
        AccountButton lo = new AccountButton("Login", this)
        {
            Location = new(ca.Location.X + ca.Size.X + (tb.Location.X / 2),ca.Location.Y),
            OnPageLoad = () =>
            {
                btn!.Textures[0] = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                page!.Controls.Add(UserName = new()
                {
                    Location = new(0, 22.ScaleInt()),
                    Size = new(page.Size.X, 31.ScaleInt()),
                    WatermarkText = "Username",
                    TextLocation = TextLocation.LineCenter,
                    AllowMultiLine = false
                });
                UserName.Textures[0] = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                UserName.LetterPress += () =>
                {
                    Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
                    if (UserName.Textures[0].handel != good.handel)
                    {
                        UserName.Textures[0] = good;
                        if (Password!.Textures[0].handel == good.handel && btn.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
                        {
                            btn.Textures[0] = good;
                        }
                        Globals.ms.TryDraw();
                    }
                    return Task.CompletedTask;
                };
                UserName.LetterRemoved += () =>
                {
                    Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
                    if (UserName.Text.Length == 0 && UserName.Textures[0].handel != bad.handel)
                    {
                        UserName.Textures[0] = bad;
                        btn.Textures[0] = bad;
                        Globals.ms.TryDraw();
                    }
                    
                    return Task.CompletedTask;
                };
                UserName.KeyPress += args =>
                {
                    if (args.Key == Keys.Tab)
                    {
                        if (args.Shift)
                        {
                            tb.Focus();
                        }
                        else
                        {
                            Password!.Focus();
                        }
                    }

                    return Task.CompletedTask;
                };
                UserName.Size = UserName.Size;
                page.Controls.Add(Password = new()
                {
                    Location = new(0, UserName.Location.Y + UserName.Size.Y + UserName.Location.Y + UserName.Location.Y),
                    Size = new(page.Size.X, UserName.Size.Y),
                    WatermarkText = "Password",
                    TextLocation = TextLocation.LineCenter,
                    AllowMultiLine = false,
                    PasswordChar = '●'
                });
                Password.Textures[0] = UserName.Textures[0];
                Password.LetterPress += () =>
                {
                    Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
                    
                    if (Password.Textures[0].handel != good.handel)
                    {
                        Password.Textures[0] = good;
                        if (UserName.Textures[0].handel == good.handel && btn.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
                        {
                            btn.Textures[0] = good;
                        }
                        Globals.ms.TryDraw();
                    }
                    return Task.CompletedTask;
                };
                Password.LetterRemoved += () =>
                {
                    Texture bad = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
                    
                    if (Password.Text.Length == 0 && Password.Textures[0].handel != bad.handel)
                    {
                        Password.Textures[0] = bad;
                        btn.Textures[0] = bad;
                        Globals.ms.TryDraw();
                    }
                    
                    return Task.CompletedTask;
                };
                Password.KeyPress += args =>
                {
                    if (args.Key == Keys.Tab && args.Shift)
                    {
                        UserName.Focus();
                    }

                    return Task.CompletedTask;
                };
                Password.Size = Password.Size;
                Password.ForceDistanceUpdate(page);
                DisplayName = null!;
                rec = null!;
                tb.Focus();
                Globals.ms.ForceUpdate();
                TryDraw();
            }
        };
        lo.Size = ca.Size;
        lo.l.Location = new((lo.Size.X / 2) - (lo.l.Size.X / 2),
            ((lo.Size.Y - lo.l.Size.Y) / 2));
        lo.l.ForceDistanceUpdate(lo);
        Form.Controls.Add(lo);
        

        #endregion
        
        Form.Controls.Add(version);

        page = new()
        {
            Location = new(tb.Location.X, ca.Location.Y + ca.Size.Y + tb.Location.X),
            BackgroundColor = Form.BackgroundColor
        };
        page.Size = new(Form.Size.X - tb.Location.X - tb.Location.X, Form.Size.Y - tb.Location.X - page.Location.Y - ca.Size.Y - tb.Location.X);
        Form.Controls.Add(page);

        btn = new(Globals.ms.TextureManager.GetTextureResource("BadTextbox.png"))
        {
            Location = new(page.Location.X, page.Location.Y + page.Size.Y + tb.Location.X),
            Size = new(page.Size.X, ca.Size.Y),
            TextureDisplay = TextureDisplay.Center
        };
        _ = ca.ToggleSelected();
        tb.Focus();
        Label sub = new(Globals.DefaultFont)
        {
            Text = "Submit",
            IgnoreHover = true
        };
        sub.Location = new((btn.Size.X / 2) - (sub.Size.X / 2),
            ((btn.Size.Y - sub.Size.Y) / 2));
        sub.ForceDistanceUpdate(btn);
        btn.Controls.Add(sub);
        Form.Controls.Add(btn);
        
        Form.Location = new((base.Size.X - Form.Size.X) / 2, (base.Size.Y - Form.Size.Y) / 2);
        Controls.Add(Form);
        btn.Clicked += BtnOnClicked;
    }

    private async Task BtnOnClicked(IRenderObject arg)
    {
        if (btn.Textures[0].handel == Globals.ms.TextureManager.GetTextureResource("BadTextbox.png").handel)
            return;
        bool s = true;
        string d = tb!.Text;
        if (d.Contains("://"))
        {
            string[] a = d.Split("://");
            d = a[1];
            if (a[0] == "http") s = false;
        }
        ServerInfo si = new()
        {
            Domain = d,
            Version = version.SelectedOption.l.Text,
            Main = false,
            Secure = s
        };
        bool g = await Globals.Luski.TryGetPublicServer(out PublicServer ps, si.Domain, si.Version, si.Secure);
        
        if (!g)
        {
            Texture b = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
            tb!.Textures[0] = b;
            btn.Textures[0] = b;
            Globals.ms.TryDraw();
            return;
        }
        
        if (g && !ps.IsLogedIn)
        {
            if (DisplayName is null)
            {
                g = await ps.Login(UserName!.Text, Password!.Text, CancellationToken.None);
            }
            else
            {
                g = await ps.CreateAccount(UserName!.Text, Password!.Text, DisplayName.Text, pfp,
                    CancellationToken.None);
            }
        }

        if (!g)
        {
            Texture b = Globals.ms.TextureManager.GetTextureResource("BadTextbox.png");
            UserName!.Textures[0] = b;
            Password!.Textures[0] = b;
            btn.Textures[0] = b;
            Globals.ms.TryDraw();
            return;
        }
        
        var l = Globals.ServerList.Servers.ToList();
        l.Add(si);
        Globals.ServerList.Servers = l.ToArray();
        Globals.ServerList.SaveSettings(Path.Combine(Globals.LuskiPath, "Servers.json"), ServerListContext.Default.ServerList);
        ServerIcon<PublicServer> ss = new ServerIcon<PublicServer>(ps);
        Globals.ms.ServerFlow.Controls.Insert(Globals.ms.ServerFlow.Controls.Length - 1, ss);
        await ss.LoadServer();
        Globals.ms.Controls.Remove(this);
        Globals.ms.TryDraw();
    }

    private Task VersionOnOptionSelected(VersionDropButton arg)
    {
        return Task.CompletedTask;
    }

    private Task VersionOnOpenStatusChanged(bool arg)
    {
        return Task.CompletedTask;
    }

    private string pfp = "";
    
    private Task RecOnFilesDroped(IRenderObject obj, string[] arg)
    {
        Console.WriteLine(arg[0]);
        if (!arg[0].ToLower().EndsWith("png")) return Task.CompletedTask;
        
        pfp = arg[0];
        if (rec!.Textures.Count() ==1 )
        {
            Texture good = Globals.ms.TextureManager.GetTextureResource("Textbox.png");
            if (UserName!.Textures[0].handel == good.handel &&
                Password!.Textures[0].handel == good.handel &&
                btn!.Textures[0].handel != good.handel && tb.Textures[0].handel == good.handel)
            {
                if (DisplayName is not null)
                {
                    if (Password!.Textures[0].handel == good.handel) btn.Textures[0] = good;
                }
                else btn.Textures[0] = good;
            }
            Texture tex = Globals.ms.TextureManager.AddTexture(File.OpenRead(arg[0]));
            tex.Unit = TextureUnit.Texture1;
            rec.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
            rec.Textures.Add(tex);
        }
        else
        {
            rec.Textures[1] = Globals.ms.TextureManager.AddTexture(File.OpenRead(arg[0]));
            rec.Textures[1].Unit = TextureUnit.Texture1;
        }
        
        Window!.TryDraw();
        return Task.CompletedTask;
    }
}